/* ftpq: delete.c

Copyright (C) 2000, 2001 Mark Tranchant

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "ftpq.h"

int delete_from_queue (int argc, char **argv) {

	int arg_ctr = 2;
	int this_job = 0;

	if (argc == 2) {
		usage ();
		exit (AB_ARGERR);
	}

	if (strcmp (argv[2], "all") == 0) {
		printf ("Deleting all jobs...");
		delete_all_jobs ();
	}
	else {
		while (arg_ctr < argc) {
			this_job = atoi (argv[arg_ctr]);
			if (this_job > 0)
				delete_job (this_job);
			arg_ctr++;
		}
	}
	return OK;
}

int delete_all_jobs (void) {

	DIR *spooldir;
	struct dirent *currfile;
	FILE *fileptr;
	char *fullpath;
	char *snppath;
	int jobid = 0;

	/* get a pointer to the spool directory */
	spooldir = opendir (SPOOLDIR);
	if (spooldir == NULL) {
		printf ("Fatal error - can't find spool directory %s!\n", SPOOLDIR);
		exit (AB_NOSPOOL);
	}

	/* read the files from the spool directory */
	while ((currfile = readdir (spooldir)) != NULL) {

		if (isdigit (currfile->d_name[0])) {
			fullpath = memalloc (strlen (SPOOLDIR) + strlen (currfile->d_name) + 1);
			sprintf (fullpath, "%s%s", SPOOLDIR, currfile->d_name);
			snppath = memalloc (strlen (fullpath) + 5);
			sprintf (snppath, "%s.snp", fullpath);

			fileptr = fileopen (fullpath, FO_READ, FO_NONEED);

			if ((jobid = get_job (fileptr)) != 0 && fileptr != NULL) {
				printf (" [%d]", jobid);
				fclose (fileptr);
				unlink (fullpath);
				unlink (snppath);
			}
			free (fullpath);
			free (snppath);
		}
	}

	printf ("...done\n");

	return OK;
}

int delete_job (int jobid) {

	char *jobfile = NULL;
	char *snppath = NULL;
	struct server *list_start;
	struct server *this_server;
	struct login *this_login;
	struct job *this_job;
	int found = 0;

	list_start = build_joblist ();

	this_server = list_start;

	while (this_server != NULL && found == 0) {
		this_login = this_server->logins;
		while (this_login != NULL && found == 0) {
			this_job = this_login->jobs;
			while (this_job != NULL && found == 0) {
				if (this_job->jobid == jobid) {
					found = 1;
					jobfile = this_job->jfile;
				}
				this_job = this_job->next;
			}
			this_login = this_login->next;
		}
		this_server = this_server->next;
	}

	if (jobfile != NULL) {
		unlink (jobfile);
		snppath = memalloc (strlen (jobfile) + 5);
		sprintf (snppath, "%s.snp", jobfile);
		unlink (snppath);
		free (snppath);
	}
	else
		printf ("Job %d not found.\n", jobid);

	free_joblist (list_start);

	return OK;
}

