/* ftpq: ftpq.c

Copyright (C) 2000, 2001 Mark Tranchant

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "ftpq.h"

int main (int argc, char **argv) {

	enum {HOST, PORT, LOGIN, PASSWD, ACCT, JOBID, RDIR, RFILE, CONFIG, WAIT_SW, DONE} state = WAIT_SW;
	enum {NOMODE, ADD, LIST, DELETE, RUN} mode = NOMODE;

	int arg_ctr = 0;

	/* Set defaults */
	gv_copy = NOCOPY;
	gv_verbose = QUIET;
	gv_port = 21;
	gv_user = getpwuid (getuid ());
	gv_host = NULL;
	gv_rdir = NULL;
	gv_rfile = NULL;
	gv_login = NULL;
	gv_passwd = NULL;
	gv_rate = 0;
	gv_lfileidx = 0;
	gv_acct = NULL;
	gv_transfer = 0L;
	gv_elapsed = 0L;

	/* No arguments - print usage and exit */
	if (argc == 1) {
		usage ();
		exit(OK);
	}

	/* Determine operating mode for this session */
	if (strcmp (argv[1], "add") == 0)
		mode = ADD;
	else if (strcmp (argv[1], "delete") == 0)
		mode = DELETE;
	else if (strcmp (argv[1], "list") == 0)
		mode = LIST;
	else if (strcmp (argv[1], "run") == 0)
		mode = RUN;
	else {
		usage ();
		exit (AB_ARGERR);
	}

	/* Process remaining arguments for add mode */

	if (mode == ADD) {
		arg_ctr = 2;
		while (arg_ctr < argc) {

			/* Snapshot switch */
			if (strcmp (argv[arg_ctr], "-s") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				gv_copy = COPY;
				state = WAIT_SW;
			}

			/* Identification switch */
			else if (strcmp (argv[arg_ctr], "-u") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = LOGIN;
			}

			/* Config switch */
			else if (strcmp (argv[arg_ctr], "-c") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = CONFIG;
			}

			/* Password switch */
			else if (strcmp (argv[arg_ctr], "-p") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = PASSWD;
			}

			/* Account switch */
			else if (strcmp (argv[arg_ctr], "-a") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = ACCT;
			}

			/* Remote path switch */
			else if (strcmp (argv[arg_ctr], "-d") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = RDIR;
			}

			/* Remote file switch */
			else if (strcmp (argv[arg_ctr], "-f") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = RFILE;
			}

			/* Remote host switch */
			else if (strcmp (argv[arg_ctr], "-h") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = HOST;
			}

			/* Remote port switch */
			else if (strcmp(argv[arg_ctr], "-P") == 0) {
				if (state != WAIT_SW) {
					printf ("Expecting parameter.\n");
					usage ();
					exit (AB_ARGERR);
				}
				state = PORT;
			}
			else if (state != DONE) {
				switch (state) {
				/* Either local file or error */
				  case WAIT_SW:
					if (gv_host != NULL && gv_login != NULL) {
						gv_lfileidx = arg_ctr;
						state = DONE;
					}
					else {
						printf ("Expecting switch.\n");
						usage ();
						exit (AB_ARGERR);
					}
					break;
				  case HOST:
					if (gv_host != NULL)
						free (gv_host);
					gv_host = memalloc (strlen (argv[arg_ctr]) + 1);
					sprintf (gv_host, "%s", argv[arg_ctr]);
					state = WAIT_SW;
					break;
				  case ACCT:
					if (gv_acct != NULL)
						free (gv_acct);
					gv_acct = memalloc (strlen (argv[arg_ctr]) + 1);
					sprintf (gv_acct, "%s", argv[arg_ctr]);
					state = WAIT_SW;
					break;
				  case RFILE:
					if (gv_rfile != NULL)
						free (gv_rfile);
					gv_rfile = memalloc (strlen (argv[arg_ctr]) + 1);
					sprintf (gv_rfile, "%s", argv[arg_ctr]);
					state = WAIT_SW;
					break;
				  case RDIR:
					if (gv_rdir != NULL)
						free (gv_rdir);
					gv_rdir = memalloc (strlen (argv[arg_ctr]) + 1);
					sprintf (gv_rdir, "%s", argv[arg_ctr]);
					state = WAIT_SW;
					break;
				  case LOGIN:
					if (gv_login != NULL)
						free (gv_login);
					gv_login = memalloc (strlen (argv[arg_ctr]) + 1);
					sprintf (gv_login, "%s", argv[arg_ctr]);
					state = WAIT_SW;
					break;
				  case PASSWD:
					if (gv_passwd != NULL)
						free (gv_passwd);
					gv_passwd = memalloc (strlen (argv[arg_ctr]) + 1);
					sprintf (gv_passwd, "%s", argv[arg_ctr]);
					state = WAIT_SW;
					break;
				  case CONFIG:
					if (read_config (argv, arg_ctr) != OK)
						exit (AB_ARGERR);
					state = WAIT_SW;
					break;
				  case PORT:
					gv_port = atoi (argv[arg_ctr]);
					if (gv_port < 1) {
						usage ();
						exit (AB_ARGERR);
					}
					state = WAIT_SW;
					break;
				/* Remaining args are multiple local files */
				  case DONE:
					break;
				  default:
					printf ("Argh - it's gone horribly wrong!\n");
					exit (AB_ARGERR);
					break;
				}
			}
			arg_ctr++;
		}

		/* Args processed: must have host, login, local file */
		if ((gv_host == NULL || gv_login == NULL || gv_lfileidx == 0) || state != DONE) {
			printf ("Missing mandatory parameter:\n");
			printf ("host: %s / login: %s\n", gv_host, gv_login);
			usage ();
			exit (AB_ARGERR);
		}

		gv_lfilectr = argc - gv_lfileidx;

		/* If multiple lfiles and rfile given, abort. */
		if (gv_rfile != NULL && gv_lfilectr > 1) {
			printf ("Can't specify remote filename with multiple local files!\n");
			usage ();
			exit (AB_ARGERR);
		}

		/* If rdir not set, make / */
		if (gv_rdir == NULL) {
			gv_rdir = memalloc (2);
			sprintf (gv_rdir, "%c", '/');
		}

		/* Now go and do it... */
		add_to_queue (argv);
	}

	else if (mode == LIST)
		list_queue ();

	else if (mode == RUN) {
		arg_ctr = 2;
		while (arg_ctr < argc) {
			if (strcmp (argv[arg_ctr], "-v") == 0)
				gv_verbose = VERBOSE;
			else if (strcmp (argv[arg_ctr], "-r") == 0) {
				gv_rate = atoi (argv[++arg_ctr]);
				if (gv_rate < 1000) {
					printf ("ftpq: silly rate.\n");
					usage ();
					exit (AB_ARGERR);
				}
			}	
			else {
				usage ();
				exit (AB_ARGERR);
			}
			arg_ctr++;
		}
		run_queue ();
	}

	else if (mode == DELETE)
		delete_from_queue (argc, argv);

	return OK;
}

