/* ftpq: ftpq.h

Copyright (C) 2000, 2001 Mark Tranchant

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* includes */
#include <ctype.h>
#include <dirent.h>
#include <errno.h>
#include <netdb.h>
#include <netinet/in.h>
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

/* useful defined constants */
#define SPOOLDIR    "/tmp/ftpq/"   /* must end in /. Check Makefile, too. */
#define HOSTRESP    2048           /* ftp server response buffer, bytes */
#define BUFSIZE     2048           /* general purpose global buffer */
#define SMTP_PORT     25

/* internal defined constants */
#define VERSION     1.04
#define OK             0
#define AB_NOFILE      1
#define AB_NOPERM      2
#define AB_FILEERR     3
#define AB_MALLOC      4
#define AB_CASEERR     5
#define AB_NOSPOOL     6
#define AB_ARGERR      7
#define FO_READ      101
#define FO_WRITE     102
#define FO_NEED      103
#define FO_NONEED    104
#define NOCOPY       201
#define COPY         202
#define VERBOSE      203
#define QUIET        204
#define NOLFILE      301
#define POS_PRELIM   401
#define POS_COMPLETE 402
#define POS_INTER    403
#define NEG_TRANS    404
#define NEG_PERM     405
#define RESP_ERROR   406
#define NOLOGIN      501
#define NOCWDROOT    502
#define NOCWD        503
#define NOPASV       504
#define NOTYPEI      505
#define NOMODES      506
#define NOSTOR       507
#define NOSERVER     508
#define NOFTPQRC     601
#define NOSECTION    602
#define MAILERR      701
#define RESP         801
#define NORESP       802
#define SEC_FAIL     901
#define SEC_PASS     902

/* Data structures and type definitions */

struct server {
	char *name;
	int port;
	struct login *logins;
	struct server *prev;
	struct server *next;
};

struct login {
	char *id;
	char *password;
	char *account;
	struct job *jobs;
	struct login *prev;
	struct login *next;
};

struct job {
	int uid;
	char *rdir;
	char *rfile;
	char *ldir;
	char *lfile;
	char *jfile;
	int jobid;
	long size;
	struct job *prev;
	struct job *next;
};

struct jfile {
	struct server server;
	struct login login;
	struct job job;
};

/* function declarations */
void usage                    (void);
FILE *fileopen                (char *, int, int);
void *memalloc                (size_t);
int read_config               (char **, int);
int fileputc                  (int, FILE *);
int add_to_queue              (char **);
int delete_from_queue         (int, char **);
int delete_all_jobs           (void);
int delete_job                (int);
int run_queue                 (void);
void list_queue               (void);
int get_job                   (FILE *);
int next_free_job             (void);
int count_jobs                (void);
struct server *add_job_node   (struct jfile, struct server *);
int isjobfile                 (char *);
struct server *build_joblist  (void);
void free_joblist             (struct server *);
void zero_jfile               (struct jfile);
int run_server                (struct server *);
int run_login                 (struct login *, struct server *, int);
int run_job                   (struct job *, struct server *, int);
int server_login              (struct server *);
int server_logout             (int);
int server_connect            (char *, int);
int get_response              (int);
int data_connect              (int, int, int, int, int, int);
int send_data                 (struct job *, int);
int send_cmd                  (char *, int, int);
int category                  (int);
int sendmail                  (struct server *, struct login *, struct job *, int);
int list_jobs                 (struct job *, int);
void sort_joblist             (struct server *);
int swap                      (struct login *, struct job *);
int get_socket                (char *, int);
int security_check            (struct job *);
int make_copy                 (char *, int, int, int);

/* global variables */
char *gv_host;
char *gv_login;
char *gv_passwd;
char *gv_acct;
char *gv_rfile;
char *gv_rdir;
int gv_lfileidx;
int gv_lfilectr;
int gv_port;
int gv_copy;
int gv_verbose;
int gv_rate;
long gv_elapsed;
long gv_transfer;
struct passwd *gv_user;
char hostresp[HOSTRESP];
char buffer[BUFSIZE];
