/* ftpq: job.c

Copyright (C) 2000, 2001 Mark Tranchant

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "ftpq.h"

/* Get the job number from a given file pointer. Return
   0 if it isn't a job file. */

int get_job (FILE *fileptr) {

	int jobid = 0;
	int scanres = 0;

	if (fileptr != NULL) {
		while (fgets (buffer, BUFSIZE, fileptr) != NULL && jobid == 0)
			scanres = sscanf (buffer, "Job: %d", &jobid);
	}

	return jobid;
}


/* Count jobs in the spool */

int count_jobs (void) {

	int job, uid, pid;
	DIR *spooldir;
	struct dirent *currfile;
	int jobs = 0;

	/* get a pointer to the spool directory */
	spooldir = opendir (SPOOLDIR);
	if (spooldir == NULL) {
		printf ("Fatal error - can't find spool directory %s!\n", SPOOLDIR);
		exit (AB_NOSPOOL);
	}

	/* read the files from the spool directory */
	while ((currfile = readdir (spooldir)) != NULL) {
		if (sscanf (currfile->d_name, "%d-%d-%d", &job, &uid, &pid) == 3) {
			/* don't count snapshot files... */
			if (strstr (currfile->d_name, ".snp") == NULL)
				jobs++;
		}
	}

	return jobs;

}

/* Find the next free job number. FIXME: this needs some file locking. */

int next_free_job (void) {

	int new_job, max_job, uid, pid;
	DIR *spooldir;
	struct dirent *currfile;

	/* get a pointer to the spool directory */
	spooldir = opendir (SPOOLDIR);
	if (spooldir == NULL) {
		printf ("Fatal error - can't find spool directory %s!\n", SPOOLDIR);
		exit (AB_NOSPOOL);
	}

	max_job = 0;
	new_job = 0;

	/* read the files from the spool directory */
	while ((currfile = readdir (spooldir)) != NULL) {
		if (sscanf (currfile->d_name, "%d-%d-%d", &new_job, &uid, &pid) == 3) {
			if (new_job > max_job) {
				max_job = new_job;
			}
		}
	}

	return max_job + 1;

}

struct server *build_joblist (void) {

	struct server *list_start = NULL;
	struct jfile this_file;
	DIR *spooldir;
	FILE *fileptr;
	struct dirent *currfile;
	char *fullpath = NULL;
	int jobid = 0;
	char line[1024];
	char buf[1024];
	int paramsize;

	this_file.server.name = NULL;
	this_file.login.id = NULL;
	this_file.login.password = NULL;
	this_file.login.account = NULL;
	this_file.job.rdir = NULL;
	this_file.job.rfile = NULL;
	this_file.job.ldir = NULL;
	this_file.job.lfile = NULL;
	this_file.job.jfile = NULL;

	/* get a pointer to the spool directory */
	spooldir = opendir (SPOOLDIR);
	if (spooldir == NULL) {
		printf ("Fatal error - can't find spool directory %s!\n", SPOOLDIR);
		exit (AB_NOSPOOL);
	}

	/* read the files from the spool directory */
	while ((currfile = readdir (spooldir)) != NULL) {

		if (isdigit (currfile->d_name[0]) && isdigit (currfile->d_name[strlen (currfile->d_name) - 1])) {
			fullpath = memalloc (strlen (SPOOLDIR) + strlen (currfile->d_name) + 2);
			sprintf (fullpath, "%s%s", SPOOLDIR, currfile->d_name);
			fileptr = fileopen (fullpath, FO_READ, FO_NONEED);
			if ((jobid = get_job (fileptr)) != 0 && fileptr != NULL ) {

				/* Start filling in the fields */
				this_file.job.jobid = jobid;
				rewind (fileptr);
				if (this_file.login.account != NULL) {
					free (this_file.login.account);
					this_file.login.account = NULL;
				}
				while (fgets (line, 1023, fileptr) != NULL) {
					if (sscanf (line, "User: %d", &this_file.job.uid));
					else if (sscanf (line, "Host: %s", buf)) {
						free (this_file.server.name);
						this_file.server.name = memalloc (strlen (buf) + 1);
						strcpy (this_file.server.name, buf);
					}
					else if (sscanf (line, "Login: %s", buf)) {
						free (this_file.login.id);
						this_file.login.id = memalloc (strlen (buf) + 1);
						strcpy (this_file.login.id, buf);
					}
					else if (sscanf (line, "Password: %s", buf)) {
						free (this_file.login.password);
						this_file.login.password = memalloc (strlen (buf) + 1);
						strcpy (this_file.login.password, buf);
					}
					else if (sscanf (line, "Account: %s", buf)) {
						free (this_file.login.account);
						this_file.login.account = memalloc (strlen (buf) + 1);
						strcpy (this_file.login.account, buf);
					}
					else if (strstr (line, "RemDir:   ")) {
						free (this_file.job.rdir);
						paramsize = strlen (line) - 11;
						this_file.job.rdir = memalloc (paramsize + 1);
						strncpy (this_file.job.rdir, &line[10], paramsize);
						this_file.job.rdir[paramsize] = '\0';
					}
					else if (strstr (line, "RemFile:  ")) {
						free (this_file.job.rfile);
						paramsize = strlen (line) - 11;
						this_file.job.rfile = memalloc (paramsize + 1);
						strncpy (this_file.job.rfile, &line[10], paramsize);
						this_file.job.rfile[paramsize] = '\0';
					}
					else if (strstr (line, "LocDir:   ")) {
						free (this_file.job.ldir);
						paramsize = strlen (line) - 11;
						this_file.job.ldir = memalloc (paramsize + 1);
						strncpy (this_file.job.ldir, &line[10], paramsize);
						this_file.job.ldir[paramsize] = '\0';
					}
					else if (strstr (line, "LocFile:  ")) {
						free (this_file.job.lfile);
						paramsize = strlen (line) - 11;
						this_file.job.lfile = memalloc (paramsize + 1);
						strncpy (this_file.job.lfile, &line[10], paramsize);
						this_file.job.lfile[paramsize] = '\0';
					}
					else if (strstr (line, "JobFile:  ")) {
						free (this_file.job.jfile);
						paramsize = strlen (line) - 11;
						this_file.job.jfile = memalloc (paramsize + 1);
						strncpy (this_file.job.jfile, &line[10], paramsize);
						this_file.job.jfile[paramsize] = '\0';
					}
					else if (sscanf (line, "Size: %ld", &this_file.job.size));
					else if (sscanf (line, "Port: %d", &this_file.server.port));
				}
				fclose (fileptr);
				list_start = add_job_node (this_file, list_start);
			}
			free (fullpath);
		}
	}

	free (this_file.login.account);
	free (this_file.server.name);
	free (this_file.login.id);
	free (this_file.login.password);
	free (this_file.login.account);
	free (this_file.job.rdir);
	free (this_file.job.rfile);
	free (this_file.job.ldir);
	free (this_file.job.lfile);
	free (this_file.job.jfile);

	sort_joblist (list_start);

	return list_start;
}

void sort_joblist (struct server *list_start) {

	struct server *this_server;
	struct login *this_login;
	struct job *this_job;
	struct job *next_job;
	int modified = 1;
	int swapped;

	this_server = list_start;

	while (this_server != NULL) {
		this_login = this_server->logins;
		while (this_login != NULL) {
			modified = 1;
			while (modified != 0) {
				modified = 0;
				this_job = this_login->jobs;
				while (this_job->next != NULL) {
					swapped = swap (this_login, this_job);
					modified += swapped;
					if (swapped == 0)
						this_job = this_job->next;
				}
			}
			this_job = this_login->jobs;

			/* Now check for obsolete jobs */
			/* Should be earliest first by now */
			while (this_job->next != NULL) {
				next_job = this_job->next;
				if (strcmp (this_job->rdir, next_job->rdir) == 0 && strcmp (this_job->rfile, next_job->rfile) == 0) {
					if (this_job->jfile != NULL) {
						printf ("ftpq: deleting obsolete job %d\n", this_job->jobid);
						unlink (this_job->jfile);
					}
					if (this_job->prev != NULL)
						this_job->prev->next = next_job;
					else
						this_login->jobs = next_job;
					free (this_job->rdir);
					free (this_job->rfile);
					free (this_job->ldir);
					free (this_job->lfile);
					free (this_job->jfile);
					free (this_job);
				}
				this_job = next_job;
			}
			this_login = this_login->next;
		}
		this_server = this_server->next;
	}
}

int swap (struct login *this_login, struct job *this_job) {

	char *this_rdir;
	char *next_rdir;
	char *this_rfile;
	char *next_rfile;
	struct job *next_job;
	int rdircmp;
	int rfilecmp;
	int swap = 0;

	next_job = this_job->next;

	this_rdir = this_job->rdir;
	next_rdir = next_job->rdir;
	this_rfile = this_job->rfile;
	next_rfile = next_job->rfile;

	rdircmp = strcmp (this_rdir, next_rdir);
	rfilecmp = strcmp (this_rfile, next_rfile);

	if (rdircmp > 0)
		swap = 1;
	else if (rdircmp == 0) {
		if (rfilecmp > 0)
			swap = 1;
		else if (rfilecmp == 0) {
			if (this_job->jobid > next_job->jobid)
				swap = 1;
		}
	}

	if (swap == 1) {

		/* Do the swap - be careful not to lose any pointers! */
		if (this_job->prev == NULL) {
			this_login->jobs = next_job;
			next_job->prev = NULL;
		}
		else {
			this_job->prev->next = next_job;
			next_job->prev = this_job->prev;
		}

		if (next_job->next != NULL) {
			next_job->next->prev = this_job;
			this_job->next = next_job->next;
		}
		else
			this_job->next = NULL;

		this_job->prev = next_job;
		next_job->next = this_job;
	}

	return swap;
}


struct server *add_job_node (struct jfile this_file, struct server *list_start) {

	struct server *this_server = NULL;
	struct server *prev_server = NULL;
	struct login *this_login = NULL;
	struct login *prev_login = NULL;
	struct job *this_job = NULL;
	struct job *prev_job = NULL;
	int match = 0;
	int firstnode = 0;

	if (list_start == NULL) {
		firstnode = 1;
	}

	if (firstnode == 0) {
		/* Try to find a matching server / port node */
		this_server = list_start;
		while (this_server != NULL && match == 0) {
			if (strcmp (this_server->name, this_file.server.name) == 0)
				if (this_server->port == this_file.server.port)
					match = 1;
			prev_server = this_server;
			this_server = this_server->next;
		}
	}

	/* No match - need new server node... */
	/* this_server will be NULL and prev_server is the end of the list */
	if (match == 0) {
		this_server = memalloc (sizeof (struct server));
		if (firstnode == 0) {
			/* Not first server node - fix pointers */
			this_server->prev = prev_server;
			prev_server->next = this_server;
		}
		else {
			/* First server node - dead end "up" */
			this_server->prev = NULL;
			list_start = this_server;
		}
		this_server->next = NULL;
		this_server->logins = NULL;
		this_server->name = memalloc (strlen (this_file.server.name) + 1);
		strcpy (this_server->name, this_file.server.name);
		this_server->port = this_file.server.port;
	}
	/* Match - use the matching value... */
	else {
		this_server = prev_server;
	}

	/* Now we look for the user details on that server... */
	this_login = this_server->logins;

	if (this_login == NULL)
		firstnode = 1;
	else
		firstnode = 0;

	match = 0;


	if (firstnode == 0) {
		/* Try to find a matching login node */
		while (this_login != NULL && match == 0) {
			if (strcmp (this_login->id, this_file.login.id) == 0) {
				/* This bit's messy - can't strcmp() a NULL */
				if (this_file.login.account == NULL && this_login->account  == NULL)
					match = 1;
				else if (this_file.login.account != NULL && this_login->account != NULL) {
					if (strcmp (this_login->account, this_file.login.account) == 0)
						match = 1;
				}
				else
						match = 0;
			}
			prev_login = this_login;
			this_login = this_login->next;
		}
	}


	/* No match - need new login node... */
	/* this_login will be NULL and prev_login is the end of the list */
	if (match == 0) {
		this_login = memalloc (sizeof (struct login));
		if (firstnode == 0) {
			/* Not first login node - fix pointers */
			this_login->prev = prev_login;
			prev_login->next = this_login;
		}
		else {
			/* First login node - dead end "up" */
			this_login->prev = NULL;
			this_server->logins = this_login;
		}
		this_login->next = NULL;
		this_login->jobs = NULL;
		this_login->id = memalloc (strlen (this_file.login.id) + 1);
		strcpy (this_login->id, this_file.login.id);
		if (this_file.login.password != NULL) {
			this_login->password = memalloc (strlen (this_file.login.password) + 1);
			strcpy (this_login->password, this_file.login.password);
		}
		else
			this_login->password = NULL;
		if (this_file.login.account != NULL) {
			this_login->account = memalloc (strlen (this_file.login.account) + 1);
			strcpy (this_login->account, this_file.login.account);
		}
		else
			this_login->account = NULL;
	}
	/* Match - use the matching value... */
	else
		this_login = prev_login;

	/* OK - we now have the server and login nodes to which to attach our job. */

	this_job = this_login->jobs;

	if (this_job == NULL)
		firstnode = 1;
	else
		firstnode = 0;

	if (firstnode == 0) {
		while (this_job != NULL) {
			prev_job = this_job;
			this_job = this_job->next;
		}
	}

	this_job = memalloc (sizeof (struct job));

	if (firstnode == 0) {
		this_job->prev = prev_job;
		prev_job->next = this_job;
	}
	else {
		this_login->jobs = this_job;
		this_job->prev = NULL;
	}

	this_job->next = NULL;
	this_job->uid = this_file.job.uid;
	this_job->jobid = this_file.job.jobid;
	this_job->rdir = memalloc (strlen (this_file.job.rdir) + 1);
	strcpy (this_job->rdir, this_file.job.rdir);
	this_job->rfile = memalloc (strlen (this_file.job.rfile) + 1);
	strcpy (this_job->rfile, this_file.job.rfile);
	this_job->ldir = memalloc (strlen (this_file.job.ldir) + 1);
	strcpy (this_job->ldir, this_file.job.ldir);
	this_job->lfile = memalloc (strlen (this_file.job.lfile) + 1);
	strcpy (this_job->lfile, this_file.job.lfile);
	this_job->jfile = memalloc (strlen (this_file.job.jfile) + 1);
	strcpy (this_job->jfile, this_file.job.jfile);
	this_job->size = this_file.job.size;

	return list_start;

}


void free_joblist (struct server *list_start) {

	struct server *this_server;
	struct server *next_server;
	struct login *this_login;
	struct login *next_login;
	struct job *this_job;
	struct job *next_job;

	this_server = list_start;

	while (this_server != NULL) {
		next_server = this_server->next;
		this_login = this_server->logins;
		free (this_server->name);
		while (this_login != NULL) {
			next_login = this_login->next;
			this_job = this_login->jobs;
			free (this_login->id);
			free (this_login->password);
			free (this_login->account);
			while (this_job != NULL) {
				next_job = this_job->next;
				free (this_job->rdir);
				free (this_job->rfile);
				free (this_job->ldir);
				free (this_job->lfile);
				free (this_job->jfile);
				free (this_job);
				this_job = next_job;
			}
			free (this_login);
			this_login = next_login;
		}
		free (this_server);
		this_server = next_server;
	}
}

