/* ftpq: list.c

Copyright (C) 2000, 2001 Mark Tranchant

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "ftpq.h"

/* Prints out spooled jobs in a tree format */

void list_queue(void) {

	struct server *list_start = NULL;
	struct server *this_server = NULL;
	struct login *this_login = NULL;
	struct job *this_job = NULL;
	long totalbytes = 0L;
	int totalfiles = 0;
	char s, l, j;

	/* Build the job list first */
	list_start = build_joblist();
	this_server = list_start;

	/* Do we need to do anything? */
	if (this_server == NULL)
		printf ("No jobs.\n");

	/* Traverse the list */
	while (this_server != NULL) {
		/* First server - use a + sign */
		if (this_server->prev == NULL)
			s = '+';

		/* Last of more than one servers */
		else if (this_server->next == NULL) {
			printf ("|\n");
			s = '`';
		}

		/* Somewhere in the middle... */
		else {
			printf ("|\n");
			s = '|';
		}

		/* Print the server heading */
		printf("%c- Server: %s:%d\n", s, this_server->name, this_server->port);
		/* Now look at logins under the current server */
		this_login = this_server->logins;
		while (this_login != NULL) {

			/* Last server - no branch to continue */
			if (this_server->next == NULL)
				s = ' ';

			/* More servers to come... */
			else
				s = '|';

			/* Last or only login */
			if (this_login->next == NULL)
				l = '`';

			/* Not last... */
			else
				l = '|';

			/* Print the login heading */
			printf ("%c  %c- User: %s  ", s, l, this_login->id);
			if (this_login->account != NULL)
				printf ("Account: %s", this_login->account);
			printf ("\n");

			/* Now to look at jobs for that login */
			this_job = this_login->jobs;
			while (this_job != NULL) {

				/* No more logins, no branch */
				/* We already know the server situation */
				if (this_login->next == NULL)
					l = ' ';

				/* Last or only job */
				if (this_job->next == NULL)
					j = '`';

				/* Not last */
				else
					j = '|';

				/* Print the job details */
				printf ("%c  %c  %c- Job %3d: ", s, l, j, this_job->jobid);
				printf ("%s", this_job->ldir);
				if (this_job->lfile[0] != '/' && this_job->ldir[strlen (this_job->ldir) - 1] != '/')
					printf ("/");
				printf ("%s", this_job->lfile);

				/* Do some really basic wrapping */
				if (strlen (this_job->ldir) + strlen (this_job->lfile) + strlen (this_job->rfile) + strlen (this_job->rdir) > 56) {
					if (this_job->next == NULL)
						j = ' ';
					printf ("\n%c  %c  %c          ", s, l, j);
				}
				printf (" to %s", this_job->rdir);
				if (this_job->rfile[0] != '/' && this_job->rdir[strlen (this_job->rdir) - 1] != '/')
					printf ("/");
				printf ("%s\n", this_job->rfile);

				/* Keep a tally of jobs and bytes */
				totalbytes += this_job->size;
				totalfiles++;
				this_job = this_job->next;
			}
			this_login = this_login->next;
		}
		this_server = this_server->next;
	}

	/* Print a grand total */
	if (list_start != NULL) {
		printf ("--------\n");
		printf ("You have %ld bytes in %d job", totalbytes, totalfiles);
		if (totalfiles > 1)
			printf ("s");
		totalfiles = count_jobs ();
		printf (" (%d job", totalfiles);
		if (totalfiles > 1)
			printf ("s");
		printf (" in total)\n");
		free_joblist (list_start);
	}
}


