/* ftpq: run.c

Copyright (C) 2000, 2001 Mark Tranchant

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "ftpq.h"

int run_queue(void) {

	struct server *list_start = NULL;
	struct server *this_server = NULL;

	list_start = build_joblist ();
	this_server = list_start;

	if (list_start == NULL)
		printf ("No jobs.\n");
	else {
		if (gv_verbose == QUIET) {
			printf ("Sending");
			if (gv_rate > 0)
				printf (" (limited to %d Bps)", gv_rate);
			printf ("...");
		}

		while (this_server != NULL) {
			run_server (this_server);
			this_server = this_server->next;
		}
		if ((gv_elapsed >> 10) > 0L && gv_rate > 0)
			printf ("\nDone! %ld bytes in %ld ms = %ld Bps\n", gv_transfer, gv_elapsed, (long) (gv_transfer / (long) (gv_elapsed >> 10))); 
		printf ("\n\n");
	}

	return OK;
}

int run_server (struct server *this_server) {

	struct login *this_login = NULL;
	int server_sockfd = 0;

	this_login = this_server->logins;

	while (this_login != NULL) {
		server_sockfd = server_login (this_server);
		if (server_sockfd != 0) {
			run_login (this_login, this_server, server_sockfd);
			server_logout (server_sockfd);
		}
		else {
			sendmail (this_server, NULL, NULL, NOSERVER);
		}
		this_login = this_login->next;
	}

	return OK;
}


int run_job (struct job *this_job, struct server *this_server, int sockfd) {

	int response;
	char *send;
	char *snpfile;
	int r[6];
	int datafd = 0;

	if (security_check (this_job) == SEC_FAIL)
		return SEC_FAIL;

	if (this_job->rdir[0] != '/') {
		response = send_cmd ("CWD /\r\n", sockfd, RESP);
		if (category (response) != POS_COMPLETE)
			return NOCWDROOT;
	}

	send = memalloc (strlen (this_job->rdir) + 7);
	sprintf (send, "CWD %s\r\n", this_job->rdir);
	response = send_cmd (send, sockfd, RESP);
	free (send);
	if (category (response) != POS_COMPLETE) {
		if (category (response) == NEG_PERM) {
			send = memalloc (strlen (this_job->rdir) + 7);
			sprintf (send, "MKD %s\r\n", this_job->rdir);
			response = send_cmd (send, sockfd, RESP);
			free (send);
			if (category (response) == POS_COMPLETE) {
				send = memalloc (strlen (this_job->rdir) + 7);
				sprintf (send, "CWD %s\r\n", this_job->rdir);
				response = send_cmd (send, sockfd, RESP);
				free (send);
				if (category (response) != POS_COMPLETE)
					return NOCWD;
			}
			else
				return NOCWD;
		}
		else
			return NOCWD;
	}

	response = send_cmd ("TYPE I\r\n", sockfd, RESP);
	if (category (response) != POS_COMPLETE)
		return NOTYPEI;

	response = send_cmd ("MODE S\r\n", sockfd, RESP);
	if (category (response) != POS_COMPLETE)
		return NOMODES;

	response = send_cmd ("PASV\r\n", sockfd, RESP);
	if (category (response) != POS_COMPLETE)
		return NOPASV;


	if ((sscanf (strchr (hostresp, '(') + 1, "%u,%u,%u,%u,%u,%u", &r[0], &r[1], &r[2], &r[3], &r[4], &r[5])) != 6)
		return NOPASV;

	datafd = data_connect (r[0], r[1], r[2], r[3], r[4], r[5]);

	if (datafd == 0)
		return NOPASV;

	send = memalloc (strlen (this_job->rfile) + 8);
	sprintf (send, "STOR %s\r\n", this_job->rfile);
	response = send_cmd (send, sockfd, RESP);
	free (send);
	if (response != 125 && response != 150) {
		close (datafd);
		return NOSTOR;
	}

	if (send_data (this_job, datafd) == -1) {
		printf ("*** File sending error!\n");
		close (datafd);
		return NOSTOR;
	}

	close (datafd);

	response = send_cmd (NULL, sockfd, RESP);
	if (category (response) == POS_COMPLETE) {
		unlink (this_job->jfile);
		snpfile = memalloc (strlen (this_job->jfile) + 5);
		sprintf (snpfile, "%s.snp", this_job->jfile);
		unlink (snpfile);
		free (snpfile);
	}
	else {
		printf ("*** Error in sending data.\n");
		return NOSTOR;
	}

	return OK;
}

int send_data (struct job *this_job, int datafd) {

	FILE *this_file = NULL;
	char *filename;
	long totalgone, bytesize, gone;
	int tosend;
	struct stat filedet;
	struct timeval *tv;
	long now, mnow, then, mthen, elapsed, rate;

	filename = memalloc (strlen (this_job->ldir) + strlen (this_job->lfile) + 2);
	sprintf (filename, "%s/%s", this_job->ldir, this_job->lfile);

	stat (filename, &filedet);
	bytesize = (off_t) filedet.st_size;

	this_file = fileopen (filename, FO_READ, FO_NONEED);

	free (filename);

	if (this_file == NULL)
		return -1;

	totalgone = 0L;

	tv = memalloc (sizeof (struct timeval));
	gettimeofday (tv, NULL);
	then = tv->tv_sec;
	mthen = tv->tv_usec >> 10;

	while (totalgone < bytesize) {
		tosend = fread (buffer, 1, (size_t) BUFSIZE, this_file);
		gone = 0L;
		while (gone < tosend) {
			gone += sendto (datafd, buffer + gone, tosend - gone, 0, NULL, 0);
			totalgone += gone;
		}
		if (gv_rate > 0) {
			gettimeofday (tv, NULL);
			now = tv->tv_sec;
			mnow = tv->tv_usec >> 10;
			elapsed = ((now - then) << 10) + (mnow - mthen);
			if (elapsed > 0L) {
				while ((rate = ((totalgone << 10) / elapsed)) > gv_rate)  {
					gettimeofday (tv, NULL);
					now = tv->tv_sec;
					mnow = tv->tv_usec >> 10; 
					elapsed = ((now - then) << 10) + (mnow - mthen);
					usleep (200000L);
				}
			}
		}
	}

	gettimeofday (tv, NULL);
	now = tv->tv_sec;
	mnow = tv->tv_usec >> 10;
	elapsed = ((now - then) << 10) + (mnow - mthen);

	gv_transfer += totalgone;
	gv_elapsed += elapsed;

	free (tv);
	fclose (this_file);
	return 0;
}


int run_login (struct login *this_login, struct server *this_server, int sockfd) {

	struct job *this_job;
	int response;
   	char *send;

	send = memalloc (strlen (this_login->id) + 8);
	sprintf (send, "USER %s\r\n", this_login->id);
	response = send_cmd (send, sockfd, RESP);
	free (send);

	if (response == 331) {                /* need password */
		send = memalloc (strlen (this_login->password) + 8);
		sprintf (send, "PASS %s\r\n", this_login->password);
		response = send_cmd (send, sockfd, RESP);
		free (send);
	}
	if (response == 332) {
		send = memalloc (strlen (this_login->account) + 8);
		sprintf (send, "ACCT %s\r\n", this_login->account);
		response = send_cmd (send, sockfd, RESP);
		free (send);
	}
	if (category (response) == POS_COMPLETE) {

		this_job = this_login->jobs;

		while (this_job != NULL) {
			response = run_job (this_job, this_server, sockfd);
			switch (response) {

			/* No problem, move on to next job */
			  case OK:
				if (gv_verbose == QUIET) {
					if (gv_copy == COPY)
						printf ("|%d| ", this_job->jobid);
					else
						printf ("[%d] ", this_job->jobid);
				}
				if ((gv_elapsed >> 10) > 0L && gv_rate > 0)
					printf ("\n%ld bytes in %ld ms = %ld Bps\n", gv_transfer, gv_elapsed, (long) (gv_transfer / (long) (gv_elapsed >> 10))); 
				this_job = this_job->next;
				break;

			/* Security check failed, give up on this job */
			  case SEC_FAIL:
				if (gv_verbose == QUIET)
					printf ("*%d* ", this_job->jobid);
				sendmail (this_server, this_login, this_job, SEC_FAIL);
				this_job = this_job->next;
				break;

			/* Can't get to /, give up on this login */
			  case NOCWDROOT:
				if (gv_verbose == QUIET)
					printf ("*%d* ", this_job->jobid);
				sendmail (this_server, this_login, this_job, NOCWDROOT);
				this_job = NULL;
				break;

			/* Can't get to final directory, give up on this job */
			  case NOCWD:
				if (gv_verbose == QUIET)
					printf ("*%d* ", this_job->jobid);
				sendmail (this_server, this_login, this_job, NOCWD);
				this_job = this_job->next;
				break;
			/* Can't set TYPE I, give up on this login */
			  case NOTYPEI:
				if (gv_verbose == QUIET)
					printf ("*%d* ", this_job->jobid);
				sendmail (this_server, this_login, this_job, NOTYPEI);
				this_job = NULL;
				break;
			/* Can't set MODE S, give up on this login */
			  case NOMODES:
				if (gv_verbose == QUIET)
					printf ("*%d* ", this_job->jobid);
				sendmail (this_server, this_login, this_job, NOMODES);
				this_job = NULL;
				break;
			/* Can't set PASV, give up on this login */
			  case NOPASV:
				if (gv_verbose == QUIET)
					printf ("*%d* ", this_job->jobid);
				sendmail (this_server, this_login, this_job, NOPASV);
				this_job = NULL;
				break;
			/* Can't store file, give up on this job */
			  case NOSTOR:
				if (gv_verbose == QUIET)
					printf ("*%d* ", this_job->jobid);
				sendmail (this_server, this_login, this_job, NOSTOR);
				this_job = this_job->next;
				break;
			/* Some other unexpected error! */
			  default:
				this_job = NULL;
				break;
			}
		}

		return OK;
	}
	else {
		sendmail (this_server, this_login, NULL, NOLOGIN);
		return NOLOGIN;
	}
}

int server_login (struct server *this_server) {

	int sockfd;

	sockfd = server_connect (this_server->name, this_server->port);

	return sockfd;

}

int server_logout (int sockfd) {

	int response;
 
	response = send_cmd ("QUIT\r\n", sockfd, RESP);

	close (sockfd);

	return OK;
}

int security_check (struct job *this_job) {

	struct stat jobdets;
	struct stat lfiledets;
	char *lfilename = NULL;

	lfilename = memalloc (strlen (this_job->ldir) + strlen (this_job->lfile) + 2);
	sprintf (lfilename, "%s/%s", this_job->ldir, this_job->lfile);

	if (stat (lfilename, &lfiledets) == -1) {
		free (lfilename);
		return SEC_FAIL;
	}

	free (lfilename);

	if (stat (this_job->jfile, &jobdets) == -1)
		return SEC_FAIL;

	if (jobdets.st_uid != lfiledets.st_uid)
		return SEC_FAIL;

	return SEC_PASS;
}
